@@ -125,7 +125,7 @@ const int SIGNAL_VALUE[MAX_SIGNAL] = { SIGHUP, SIGINT, SIGQUIT,
 
 
 //return ta-tb in microseconds (no overflow checks!)
-inline long timediff(const struct timespec *ta,const struct timespec *tb) {
+long timediff(const struct timespec *ta,const struct timespec *tb) {
     unsigned long us = (ta->tv_sec-tb->tv_sec)*1000000 + (ta->tv_nsec/1000 - tb->tv_nsec/1000);
     return us;
 }
@@ -376,7 +376,7 @@ void Child_Done(int sig)
    caught_child = waitpid(-1, NULL, WNOHANG);
    if (verbose)
    {
-      printf("Caught child process: %d\n", (int) caught_child);
+      printf("Caught child process: %ld\n", (long int) caught_child);
       printf("%d\n", errno);
    }
    // If this was the one process we were watching, we can quit now.
@@ -601,6 +601,8 @@ CHILD *Remove_Child(CHILD *all_children, pid_t old_pid)
     {
         if (current->child == old_pid)
         {
+           current->monitor = 0;
+           current->child = 0;
            if (previous)
                previous->next = current->next;
            else
@@ -671,10 +673,11 @@ CHILD *Find_Child_PIDs(CHILD *all_children, pid_t parent_pid)
     char line[256];
     pid_t new_ppid;
     int current_pid;
+    CHILD *my_children = NULL;
 
     proc = opendir("/proc");
     if (! proc)
-       return all_children;
+       return NULL;
 
     proc_entry = readdir(proc);
     while (proc_entry)
@@ -694,7 +697,7 @@ CHILD *Find_Child_PIDs(CHILD *all_children, pid_t parent_pid)
                  {
                      sscanf(proc_entry->d_name, "%d", &current_pid);
                      if (! Find_Child(all_children, current_pid) )
-                        all_children = Add_Child(all_children, current_pid);
+                        my_children = Add_Child(all_children, current_pid);
                  }
                  found = TRUE;
              }
@@ -707,7 +710,7 @@ CHILD *Find_Child_PIDs(CHILD *all_children, pid_t parent_pid)
         proc_entry = readdir(proc);
     }   // done reading proc file system
     closedir(proc);
-    return all_children;
+    return my_children;
 }
 
 
@@ -734,33 +737,33 @@ void *Monitor_Children(void *all_data)
       while (current)
       {
          // First see if the child process is still running. If not,
-         // we can remote its node.
+         // we can remove its node.
          if (current->child)
          {
              char filename[PROC_FILENAME + 1];
              DIR *child_directory;
-             snprintf(filename, PROC_FILENAME, "/proc/%d", current->child);
+             snprintf(filename, PROC_FILENAME, "/proc/%ld", (long int) current->child);
              child_directory = opendir(filename);
              if (child_directory)
                 closedir(child_directory);
              else
              {
                 if (verbose)
-                  printf("Child process %d done, cleaning up.\n",
-                          (int) current->child);
+                  printf("Child process %ld done, cleaning up.\n",
+                          (long int) current->child);
                 all_children = Remove_Child(all_children, current->child);
              }
          }  // end of clean up children processes no longer running
 
          // The child process is still running, but it might not have
          // a monitor. Create a new monitoring process.
-         if ( (current->child) && (! current->monitor) )
+         else if ( (current->child) && (! current->monitor) )
          {
               pid_t returned_pid;
 
               if (verbose)
-                 printf("Creating monitoring process for %d\n",
-                        (int) current->child);
+                 printf("Creating monitoring process for %ld\n",
+                        (long int) current->child);
 
               returned_pid = fork();
               if (returned_pid > 0)
@@ -774,7 +777,7 @@ void *Monitor_Children(void *all_data)
                  char limit_amount[16];
                  char process_identifier[16];
                  snprintf(limit_amount, 16, "%d", (int) program_data->limit);
-                 snprintf(process_identifier, 16, "%d", current->child);
+                 snprintf(process_identifier, 16, "%ld", (long int) current->child);
                  if (verbose)
                     printf("Starting monitor with: %s -l %s -p %s -z -m\n",
                             program_data->this_program, limit_amount,
@@ -790,10 +793,10 @@ void *Monitor_Children(void *all_data)
               }
               
          }    // end of creating a new monitor
-         if (verbose)
+         if ( (verbose) && (! quitting) )
          {
-             printf("Watching child: %d with %d\n", 
-                   (int) current->child, (int) current->monitor);
+             printf("Watching child: %ld with %ld\n", 
+                   (long int) current->child, (long int) current->monitor);
          }
          current = current->next;
       }
@@ -1083,7 +1086,7 @@ int main(int argc, char **argv) {
                     lazy = TRUE;
                     pid_ok = TRUE;
                     if (verbose)
-                      printf("Throttling process %d\n", (int) pid);
+                      printf("Throttling process %ld\n", (long int) pid);
                  }
                  else    // parent cpulimit process which can quit
                    exit(0);
